package com.dt.intercepter;


import com.dt.config.ApiException;
import com.dt.config.CommonContants;
import com.dt.config.NonLoginRequired;
import com.dt.util.TokenUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;

@Component
@Slf4j
public class AuthIntercepter implements HandlerInterceptor {


    @Autowired
    private StringRedisTemplate stringRedisTemplate;



    public final static  String Token="token";

    public AuthIntercepter() {

    }

    public boolean preHandle(HttpServletRequest request, HttpServletResponse response,Object handle)
    {
        String token  = request.getHeader(Token);

        String requestURI = request.getRequestURI();
        if (requestURI.equals("/user/login")){
            return true;
        }
        try {
            if (!(handle instanceof HandlerMethod))
            {
                return true;
            }
            HandlerMethod handlerMethod=(HandlerMethod)handle;
            Method method = handlerMethod.getMethod();
            NonLoginRequired loginRequired = method.getAnnotation(NonLoginRequired.class);
            if (loginRequired!=null)
            {
                return true;
            }
            if (StringUtils.isEmpty(token))
            {
                log.info("token为空");
                throw new Exception("token错误,请重新登录");
            }
            Long userId = TokenUtils.getUserId(token);
            log.info("userId",userId);
            String s = stringRedisTemplate.opsForValue().get(userId);
            if (s.equals(token))
            {
                log.info("*************用户不存在");
                throw new ApiException(101,"token错误");
            }
            CommonContants.local.set(userId.toString());
        } catch (Exception e) {
            log.info("调用了异常通知",e.getMessage());
            throw new RuntimeException(e);
        }
        return true;
    }
}
