package com.dt.util;

/**
 * 增强版业务异常类
 * 包含错误码、错误信息、可选数据载荷
 */
public class RRException extends RuntimeException {


    private final int code;
    private final String msg;
    private transient Object data; // transient避免序列化问题

    // 常用构造方法
    public RRException(int code, String msg) {
        super(msg);
        this.code = code;
        this.msg = msg;
    }

    public RRException(int code, String msg, Object data) {
        super(msg);
        this.code = code;
        this.msg = msg;
        this.data = data;
    }

    public RRException(int code, String msg, Throwable cause) {
        super(msg, cause);
        this.code = code;
        this.msg = msg;
    }

    // 快速创建异常的静态工厂方法
    public static RRException of(int code, String msg) {
        return new RRException(code, msg);
    }

    public static RRException of(int code, String msg, Object data) {
        return new RRException(code, msg, data);
    }

    // Getter 方法
    public int getCode() {
        return code;
    }

    public String getMsg() {
        return msg;
    }

    public Object getData() {
        return data;
    }

    @Override
    public String toString() {
        return "RRException{" +
                "code=" + code +
                ", msg='" + msg + '\'' +
                ", data=" + data +
                '}';
    }
}